package cz.fura;

import junit.framework.TestCase;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.BeansException;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceEditor;
import org.springframework.validation.BindingResult;
import org.springframework.validation.DataBinder;

/**
 * Spring introspection demo test.
 *
 * @author Jan Novotn, FG Forrest a.s. (c) 2007
 * @version $Id: $
 */
public class SpringIocTest extends TestCase {

	public void testBeanWrapperPopulation() throws Exception {
		//create custom pojo instance
		User myJavaBean = new User();
		//create wrapper
		BeanWrapper wrapper = new BeanWrapperImpl(myJavaBean);

		//use simple population
		wrapper.setPropertyValue("login", "novoj");
		wrapper.setPropertyValue("password", "heslo");
		//check it
		assertEquals("novoj", myJavaBean.getLogin());
		assertEquals("heslo", myJavaBean.getPassword());

		//use collected values population
		MutablePropertyValues pvs = new MutablePropertyValues();
		pvs.addPropertyValue("address.street", "U eky 1");
		pvs.addPropertyValue("address.town", "Krlovec");
		pvs.addPropertyValue("address.country", "esk republika");
		wrapper.setPropertyValues(pvs);
		//check it
		assertEquals("U eky 1", myJavaBean.getAddress().getStreet());
		assertEquals("Krlovec", myJavaBean.getAddress().getTown());
		assertEquals("esk republika", myJavaBean.getAddress().getCountry());

		//populate list data
		wrapper.setPropertyValue("tags[0]", "redTeam");
		wrapper.setPropertyValue("tags[1]", "management");
		//check it
		assertTrue(myJavaBean.getTags().contains("redTeam"));
		assertTrue(myJavaBean.getTags().contains("management"));

		//populate even map properties
		wrapper.setPropertyValue("properties[blogger]", Boolean.FALSE);
		wrapper.setPropertyValue("properties[degree]", "university");
		//check it
		assertEquals(Boolean.FALSE, myJavaBean.getProperties().get("blogger"));
		assertEquals("university", myJavaBean.getProperties().get("degree"));

		//let's look at more difficult types - PropertyEditors come handy
		wrapper.setPropertyValue("email", "Otec Fura (novotnaci@gmail.com)");
		//check it
		assertEquals("Otec Fura", myJavaBean.getEmail().getName());
		assertEquals("novotnaci@gmail.com", myJavaBean.getEmail().getAddress());
		//will throw exception
		try {
			wrapper.setPropertyValue("notExistingProperty", "doesn't matter");
			fail("Exception expected!");
		} catch(BeansException ex) {
			//that is ok
		}
	}

	public void testDataBinder() throws Exception {
		//create custom pojo instance
		User myJavaBean = new User();
		myJavaBean.setLogin("novoj");
		myJavaBean.setPassword("password");
		myJavaBean.setEmail(new EmailAddress("novotnaci@gmail.com", "Otec Fura"));
		myJavaBean.setProperty("degree", "high school");
		myJavaBean.setProperty("blogger", Boolean.TRUE);
		//create data binder
		DataBinder binder = new DataBinder(myJavaBean);
		binder.setIgnoreUnknownFields(true);
		binder.setIgnoreInvalidFields(false);
		binder.setDisallowedFields(new String[] {"login", "address.*"});
		binder.setRequiredFields(new String[] {"password", "properties[degree]"});
		//create input values map
		MutablePropertyValues pvs = new MutablePropertyValues();
		//will be ignored - is in disallowed fields
		pvs.addPropertyValue("login", "newLogin");
		//will be populated
		pvs.addPropertyValue("password", "newPassword");
		//would trigger exception if ignoreUnknownfields == false
		pvs.addPropertyValue("nonExistingField", "doesn't matter");
		//will result in error in binding ressult
		pvs.addPropertyValue("email", Boolean.FALSE);
		//will result in required error in result if commented out
		//pvs.addPropertyValue("properties[degree]", "university");
		//bind values at one single shot
		binder.bind(pvs);
		//check errors
		BindingResult result = binder.getBindingResult();		
		assertEquals(2, result.getErrorCount());
		assertEquals("novoj", myJavaBean.getLogin());
		assertEquals("newPassword", myJavaBean.getPassword());
		assertEquals("Otec Fura", myJavaBean.getEmail().getName());
		assertEquals("novotnaci@gmail.com", myJavaBean.getEmail().getAddress());
		assertEquals("high school", myJavaBean.getProperties().get("degree"));
	}

	public void testCustomResourceLoader() throws Exception {
		//create custom pojo instance
		User myJavaBean = new User();
		//create wrapper
		BeanWrapper wrapper = new BeanWrapperImpl(myJavaBean);
		//bind custom resource
		wrapper.setPropertyValue("photo", "corpWeb:/img/u/logo_title.gif");
		//because default resource editor doesn't know corpWeb location handling it will use classpath resource
		assertNotNull(myJavaBean.getPhoto());
		//that mean, that such resource won't be accessible
		assertFalse(myJavaBean.getPhoto().exists());
		//but when we register custom resource loader being able to process corpWeb location situation changes
		wrapper.registerCustomEditor(Resource.class, new ResourceEditor(new CustomResourceLoader()));
		//bind custom resource
		wrapper.setPropertyValue("photo", "corpWeb:/img/u/logo_title.gif");
		//check that photo exists
		assertNotNull(myJavaBean.getPhoto());
		assertTrue(myJavaBean.getPhoto().exists());
	}

}
